
# q-expansion of newform 6200.2.a.j, downloaded from the LMFDB on 11 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6200
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [0], [0], [0], [2], [-3], [1], [0], [6], [1], [-3], [-11], [-9], [-10], [1], [13], [8], [-2], [3], [7], [2], [-9], [4], [-14], [-3], [-12], [-2], [-5], [4], [-8], [9], [-21], [-10], [13], [-22], [4], [16], [3], [-12], [-12], [-14], [4], [14], [-6], [16], [12], [-11], [6], [6], [4], [-10], [-28], [-2], [-6], [13], [6], [-18], [-19], [-1], [-26], [-14], [16], [-13], [21], [-32], [4], [-11], [8], [15], [10], [24], [3], [-10], [21], [-5], [-10], [14], [28], [-14], [23], [-5], [-3], [-2], [-9], [34], [2], [10], [20], [-28], [-8], [32], [-9], [0], [-10], [-14], [20], [-21], [-3], [10], [24], [-34], [-14], [-4], [38], [-42], [24], [-16], [24], [40], [-14], [15], [-28], [32], [20], [-24], [41], [33], [12], [-23], [9], [11], [-17], [-4], [9], [-2], [-34], [8], [12], [-32], [-50], [11], [20], [-25], [28], [-5], [-46], [28], [-26], [10], [-5], [-20], [19], [-5], [-18], [55], [-12], [-12], [-32], [25], [20], [58], [-5], [-42], [-4], [12], [-11], [-8], [4], [-18], [-15], [47], [-16], [31], [60], [40], [-16], [48], [-24], [34], [-26], [40], [-35], [44], [37], [-54], [37], [-30], [-54], [-8], [-12], [50], [34], [21], [4], [-48], [38], [2], [-56], [63], [30], [42], [-10], [-64], [52], [6], [-20], [-28], [-22], [22], [-45], [42], [-26], [-10], [52], [53], [-41], [15], [27], [-33], [33], [-16], [62], [-2], [-48], [-7], [32], [38], [63], [-2], [38], [-32], [31], [4], [30], [-28], [-66], [-10], [-37], [68], [-29], [20], [13], [16], [20], [-72], [-67], [14], [4], [-14], [-24], [40], [-71], [-20], [11], [-16], [17], [-18], [38], [-66], [10], [75], [64], [77], [32], [-53], [54], [43], [68], [-8], [8], [-8], [-2], [26], [18], [12], [66], [10], [-48], [-2], [29], [-68], [39], [22], [-78], [-38], [-4], [-62], [20], [-52], [-12], [53], [80], [-57], [-70], [66], [-62], [33], [-20], [54], [-39], [-66], [49], [37], [-50], [56], [-26], [-72], [0], [-19], [16], [45], [-52], [36], [-30], [38], [77], [38], [-26], [-2], [39], [-30], [-15], [-59], [-38], [18], [4], [-70], [-14], [-60], [-6], [-22], [-66], [74], [2], [56], [-92], [-70], [-43], [-33], [-31], [48], [-40], [60], [-17], [-30], [48], [-18], [-73], [-55], [-93], [42], [-72], [-38], [30], [-13], [-75], [28], [-78], [-2], [58], [35], [53], [-80], [14], [54], [34], [12], [-39], [24], [-12], [-4], [-75], [-4], [-46], [66], [42], [19], [5], [-84], [-39], [90], [21], [-32], [69], [-66], [0], [36], [-61], [-19], [-78], [-25], [83], [-15], [-28], [32], [75], [57], [-36], [3], [-20], [-21], [36], [-80], [-43], [2], [-48], [31], [-28], [-30], [14], [34], [-50], [-72], [10], [-7], [-50], [-22], [55], [-62], [55], [-66], [-5], [-8], [58], [-81], [-66], [-42], [6], [32], [-28], [37], [-91], [21], [-22], [67], [36], [50], [-78], [-6]]
